#!/usr/bin/ksh
#set -x
#
# $Id: check_ramsan.sh 253 2012-03-18 22:22:12Z u09422fra $
#
# TMS RamSan health monitoring plugin for Nagios 
#
# Paths 
SNMPGET="/usr/bin/snmpget"
SNMPWALK="/usr/bin/snmpwalk"

# Nagios return codes and strings
R_OK=0;       S_OK="OK"
R_WARNING=1;  S_WARNING="WARNING"
R_CRITICAL=2; S_CRITICAL="CRITICAL"
R_UNKNOWN=3;  S_UNKNOWN="UNKNOWN"
R_ALL=0;      S_ALL=""
S_PREFIX="RamSan"

# SNMP OIDs
overallState='.1.3.6.1.4.1.8378.10.1.3.0'
fanState='.1.3.6.1.4.1.8378.10.1.6.0.1.6'
temperatureState='.1.3.6.1.4.1.8378.10.1.6.1.1.6'
powerState='.1.3.6.1.4.1.8378.10.1.6.2.1.6'
fcState='.1.3.6.1.4.1.8378.10.2.1.5'

# fcState:
# "FC State which indicates the link state of the specified port:
#    None     - No state available
#    No Light - Nothing is plugged into the fc port
#    Offline  - A device is plugged into the fc port, but the linkup
#               procedure has not completed successfully
#    Online   - The port is online and functional from a link point of view
#    Disabled - The port has been disabled, likely because the system is
#               restoring memory from disk, or preparing to poweroff."

# Variables
OID=""
PROGNAME=$(basename $0)
SENSORNO=0
SNMPGET_ARGS="-On -Oa -v 2c -c public -t 15"
SUFFIX=""

if [ ! -x ${SNMPGET} ]; then
    echo "UNKNOWN: ${SNMPGET} not found or not executable."
    exit ${R_UNKNOWN}
fi
if [ ! -x ${SNMPWALK} ]; then
    echo "UNKNOWN: ${SNMPWALK} not found or not executable."
    exit ${R_UNKNOWN}
fi

# Functions
# Display usage
usage() {
    echo "Usage: ${PROGNAME} [-h] -H hostname -C check"
    echo "    -h display help"
    echo "    -H run check on this hostname"
    echo "    -C run one of the following checks:"
    echo "        status, fanStatus, tempStatus, powerStatus, fcStatus"
    echo ""
    exit ${R_UNKNOWN}
}

# Parse parameters
# Get command line options
[ $# -eq 0 ] && usage
while getopts  "hC:H:" OPTION; do
    case ${OPTION} in
        h) usage ;;
        C) if [[ ! -z ${OPTARG} ]]; then
               CHECK=${OPTARG}
           else
               usage
           fi
        ;;
        H) if [[ ! -z ${OPTARG} ]]; then
               CHECK_HOST=${OPTARG}
           else
               usage
           fi
        ;;
        *) usage ;;
    esac
done
[[ -z ${CHECK} || -z ${CHECK_HOST} ]] && usage

case ${CHECK} in
    status)
        # Get RamSan health status
        S_overallState=`${SNMPGET} ${SNMPGET_ARGS} ${CHECK_HOST} ${overallState}`
        S_overallState="${S_overallState##*: }"
        S_overallState="${S_overallState//\"/}"
        S_overallState="${S_overallState//\./}"
        if [[ "${S_overallState}" == "Good" ]]; then
            S_ALL="${S_PREFIX} ${S_OK} - Status: ${S_overallState}"
            R_ALL=${R_OK}
        elif [[ "${S_overallState}" != "Good" ]]; then
            S_ALL="${S_PREFIX} ${S_CRITICAL} - Status: ${S_overallState}"
            R_ALL=${R_CRITICAL}
        else
            S_ALL="${S_PREFIX} ${S_UNKNOWN} - Status: ${S_overallState}"
            R_ALL=${R_UNKNOWN}
        fi
    ;;
    fanStatus|tempStatus|powerStatus|fcStatus)
        case ${CHECK} in
            fanStatus)    OID="${fanState}" ;;
            tempStatus)   OID="${temperatureState}" ;;
            powerStatus)  OID="${powerState}" ;;
            fcStatus)     OID="${fcState}" ;;
        esac
        # Get status for each subsystem
        S_ALL="${S_OK}"
        S_TMP_GOOD=""
        S_TMP_FAULT=""
        ${SNMPWALK} ${SNMPGET_ARGS} ${CHECK_HOST} ${OID} 2>/dev/null | \
        while read OID SIGN TYPE STATUS; do 
            ITEMNO="${OID##*.}"
            case ${CHECK} in
                fanStatus)
                    S_PREFIX="FAN"
                    ITEMNAME="Fan$((ITEMNO+1))"
                    STATUS="${STATUS//\"/}"
                    STATUS="${STATUS//\./}"
                    if [[ "${STATUS}" != "GOOD" ]]; then
                        R_ALL="${R_CRITICAL}"
                        S_ALL="${S_CRITICAL}"
                        S_TMP_FAULT="${S_TMP_FAULT}${ITEMNAME}(${STATUS}):"
                    else
                        S_TMP_GOOD="${S_TMP_GOOD}${ITEMNAME}(${STATUS}):"
                    fi
                ;;
                tempStatus)
                    S_PREFIX="TEMP"
                    ITEMNAME="Temp$((ITEMNO+1))"
                    STATUS="${STATUS//\"/}"
                    STATUS="${STATUS//\./}"
                    if [[ "${STATUS}" != "GOOD" ]]; then
                        R_ALL="${R_CRITICAL}"
                        S_ALL="${S_CRITICAL}"
                        S_TMP_FAULT="${S_TMP_FAULT}${ITEMNAME}(${STATUS}):"
                    else
                        S_TMP_GOOD="${S_TMP_GOOD}${ITEMNAME}(${STATUS}):"
                    fi
                ;;
                powerStatus)
                    S_PREFIX="POWER"
                    ITEMNAME="Power$((ITEMNO+1))"
                    STATUS="${STATUS//\"/}"
                    STATUS="${STATUS//\./}"
                    if [[ "${STATUS}" != "GOOD" ]]; then
                        R_ALL="${R_CRITICAL}"
                        S_ALL="${S_CRITICAL}"
                        S_TMP_FAULT="${S_TMP_FAULT}${ITEMNAME}(${STATUS}):"
                    else
                        S_TMP_GOOD="${S_TMP_GOOD}${ITEMNAME}(${STATUS}):"
                    fi
                ;;
                fcStatus)
                    S_PREFIX="FC"
                    ITEMNAME="FC$((ITEMNO+1))"
                    STATUS="${STATUS//\"/}"
                    STATUS="${STATUS//\./}"
                    if [[ "${STATUS}" != "ONLINE" ]]; then
                        R_ALL="${R_CRITICAL}"
                        S_ALL="${S_CRITICAL}"
                        S_TMP_FAULT="${S_TMP_FAULT}${ITEMNAME}(${STATUS}):"
                    else
                        S_TMP_GOOD="${S_TMP_GOOD}${ITEMNAME}(${STATUS}):"
                    fi
                ;;
            esac
        done 

        S_TMP_FAULT="${S_TMP_FAULT%:}"
        S_ALL="${S_PREFIX} ${S_ALL} - ${S_TMP_FAULT:=All items optimal.}"
    ;;
    *) usage ;;
esac

# Return
echo "${S_ALL}"
exit ${R_ALL}

exit 0
#
## EOF
